/**
 * vue-stator v0.0.6
 * (c) 2019
 * - Jonas Galvez (@galvez)
 * - Pim (@pimlie)
 * - All the amazing contributors
 * @license MIT
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue'], factory) :
  (global = global || self, factory(global.VueStator = {}, global.Vue));
}(this, function (exports, Vue) { 'use strict';

  Vue = Vue && Vue.hasOwnProperty('default') ? Vue['default'] : Vue;

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);

    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      if (enumerableOnly) symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
      keys.push.apply(keys, symbols);
    }

    return keys;
  }

  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};

      if (i % 2) {
        ownKeys(source, true).forEach(function (key) {
          _defineProperty(target, key, source[key]);
        });
      } else if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
      } else {
        ownKeys(source).forEach(function (key) {
          Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
      }
    }

    return target;
  }

  function _dumpState(storage, namespace, data) {
    if (!storage) {
      return;
    }

    storage.setItem("vue-stator:".concat(namespace), JSON.stringify(data));
  }

  function _loadState(storage, namespace, onSuccess, onError) {
    if (!storage) {
      return;
    }

    try {
      var data = JSON.parse(storage.getItem("vue-stator:".concat(namespace)));

      if (data) {
        onSuccess(data);
      }
    } catch (err) {
      // eslint-disable-next-line no-console
      console.error("[vue-stator] error in _loadState(".concat(Array.prototype.slice.call(arguments).join(', '), ")"), err);
    }
  }

  var _watcher;

  function _subscribe(ctx, key, callback) {
    if (!_watcher) {
      _watcher = new Vue();
    }

    return _watcher.$watch(function () {
      return ctx.$state[key];
    }, function (data) {
      return callback(data);
    }, {
      deep: true
    });
  } // Public helpers

  function $set() {
    Vue.set.apply(Vue, arguments);
  }
  function $delete() {
    Vue.delete.apply(Vue, arguments);
  }
  function mapState(namespace, properties) {
    if (Array.isArray(namespace)) {
      return namespace.reduce(function (mapped, prop) {
        mapped[prop] = {
          get: function get() {
            return this.$state[prop];
          },
          set: function set(value) {
            this.$state[prop] = value;
            return this.$state[prop];
          }
        };
        return mapped;
      }, {});
    }

    return properties.reduce(function (mapped, prop) {
      mapped[prop] = {
        get: function get() {
          return this.$state[namespace][prop];
        },
        set: function set(value) {
          this.$state[namespace][prop] = value;
          return this.$state[prop];
        }
      };
      return mapped;
    }, {});
  }
  function mapActions(namespace, properties) {
    if (Array.isArray(namespace)) {
      return namespace.reduce(function (mapped, prop) {
        mapped[prop] = function () {
          var _this$$actions;

          return (_this$$actions = this.$actions)[prop].apply(_this$$actions, arguments);
        };

        return mapped;
      }, {});
    }

    return properties.reduce(function (mapped, prop) {
      mapped[prop] = function () {
        var _this$$actions$namesp;

        return (_this$$actions$namesp = this.$actions[namespace])[prop].apply(_this$$actions$namesp, arguments);
      };

      return mapped;
    }, {});
  }
  function mapGetters(namespace, getters) {
    if (Array.isArray(namespace)) {
      return namespace.reduce(function (mapped, getter) {
        mapped[getter] = function () {
          return this.$getters[getter];
        };

        return mapped;
      }, {});
    }

    return getters.reduce(function (mapped, getter) {
      mapped[getter] = function () {
        return this.$getters[namespace][getter];
      };

      return mapped;
    }, {});
  }
  function createStore(_ref) {
    var ctx = _ref.ctx,
        state = _ref.state,
        actions = _ref.actions,
        getters = _ref.getters,
        hydrate = _ref.hydrate,
        localStorage = _ref.localStorage,
        sessionStorage = _ref.sessionStorage;
    var initialState = state(ctx);

    if (ctx._isVue) {
      ctx = {};
    }

    if (localStorage) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        var _loop = function _loop() {
          var namespace = _step.value;

          _loadState(window.localStorage, state, function (data) {
            return $set(ctx.$state, namespace, data);
          });

          _subscribe(ctx, namespace, function (snapshot) {
            return _dumpState(window.localStorage, namespace, snapshot);
          });
        };

        for (var _iterator = localStorage[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          _loop();
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    if (sessionStorage) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        var _loop2 = function _loop2() {
          var namespace = _step2.value;

          _loadState(window.sessionStorage, state, function (data) {
            return $set(ctx.$state, namespace, data);
          });

          _subscribe(ctx, namespace, function (snapshot) {
            return _dumpState(window.sessionStorage, namespace, snapshot);
          });
        };

        for (var _iterator2 = sessionStorage[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          _loop2();
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }

    if (hydrate) {
      ctx.$state = hydrate(initialState);
    } else {
      ctx.$state = Vue.observable(initialState);
    }

    if (getters) {
      ctx.$getters = {};
      registerGetters(ctx, getters);
    }

    if (actions) {
      ctx.$actions = {};
      registerActions(ctx, actions);
    }

    return ctx;
  } // Registration helpers

  function registerGetters(ctx, getters) {
    ctx.$getters = {};

    var _loop3 = function _loop3() {
      var key = _Object$keys[_i];

      if (typeof getters[key] !== 'function') {
        ctx.$getters[key] = {};

        for (var _i2 = 0, _Object$keys2 = Object.keys(getters[key]); _i2 < _Object$keys2.length; _i2++) {
          var subKey = _Object$keys2[_i2];
          Object.defineProperty(ctx.$getters[key], subKey, {
            get: function get() {
              return getters[key](ctx.$state[key], ctx.$getters[key], ctx.$state);
            }
          });
        }

        return "continue";
      }

      Object.defineProperty(ctx.$getters, key, {
        get: function get() {
          return getters[key](ctx.$state, ctx.$getters);
        }
      });
    };

    for (var _i = 0, _Object$keys = Object.keys(getters); _i < _Object$keys.length; _i++) {
      var _ret = _loop3();

      if (_ret === "continue") continue;
    }

    return ctx.$getters;
  }
  function registerActions(ctx, actions) {
    ctx.$actions = {};

    var _loop4 = function _loop4() {
      var key = _Object$keys3[_i3];

      if (typeof actions[key] !== 'function') {
        ctx.$actions[key] = {};

        var _loop5 = function _loop5() {
          var subKey = _Object$keys4[_i4];

          ctx.$actions[key][subKey] = function () {
            var _actions$key;

            for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
              args[_key] = arguments[_key];
            }

            return (_actions$key = actions[key])[subKey].apply(_actions$key, [ctx, ctx.$state[key]].concat(args));
          };
        };

        for (var _i4 = 0, _Object$keys4 = Object.keys(actions[key]); _i4 < _Object$keys4.length; _i4++) {
          _loop5();
        }

        return "continue";
      }

      ctx.$actions[key] = function () {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return actions[key].apply(actions, [ctx, ctx.$state].concat(args));
      };
    };

    for (var _i3 = 0, _Object$keys3 = Object.keys(actions); _i3 < _Object$keys3.length; _i3++) {
      var _ret2 = _loop4();

      if (_ret2 === "continue") continue;
    }

    return ctx.$actions;
  } // Vanilla Installer

  function lazyInject(Vue, key, setter) {
    // Check if plugin not already installed
    var installKey = '__stator_lazy_' + key + '_installed__';

    if (Vue[installKey]) {
      return;
    }

    Vue[installKey] = true; // Call Vue.use() to install the plugin into vm

    Vue.use(function () {
      if (!Vue.prototype.hasOwnProperty(key)) {
        Object.defineProperty(Vue.prototype, key, {
          configurable: true,
          get: function get() {
            this.$root.$options[key] = setter(this);
            Object.defineProperty(Vue.prototype, key, {
              get: function get() {
                return this.$root.$options[key];
              }
            });
            return this.$root.$options[key];
          }
        });
      }
    });
  }

  var plugin = {
    install: function install(Vue) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      lazyInject(Vue, '_stator', function (vm) {
        return createStore(_objectSpread2({
          ctx: vm
        }, options));
      });
      lazyInject(Vue, '$state', function (vm) {
        return vm._stator.$state;
      });
      lazyInject(Vue, '$getters', function (vm) {
        return vm._stator.$getters;
      });
      lazyInject(Vue, '$actions', function (vm) {
        return vm._stator.$actions;
      });
    }
  };

  exports.$delete = $delete;
  exports.$set = $set;
  exports._subscribe = _subscribe;
  exports.createStore = createStore;
  exports.mapActions = mapActions;
  exports.mapGetters = mapGetters;
  exports.mapState = mapState;
  exports.plugin = plugin;
  exports.registerActions = registerActions;
  exports.registerGetters = registerGetters;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
